(function($) {
  $(document).on('Filter:present', function(event){
    var section = $(event.target);
    var clearLink = $('.js-clear-filters-link', section);
    var facets = $('.js-filter__facet', section);
    var subFacets = $('.js-filter__sub-facet', section);

    var facetsAndSubFacets = facets.toArray().concat(subFacets.toArray());
    var facetsOrSubFacetsChecked = _.any(facetsAndSubFacets, function(facet) {
      return $(facet).is(':checked');
    });
    if(facetsOrSubFacetsChecked) { clearLink.show(); }

    // Facet changed
    facets.on('change', function(event) {
      var facet = $(event.target);
      updateSubFacets(facet);
      if(facet.attr('checked')) {
        facet.trigger('expandFilters');
        clearLink.show();
      }
      hideClearLink();
    });

    function updateSubFacets(facet) {
      var filterSet = facet.closest('.js-filter');
      var subFacetsInFilterSet = $('.js-filter__sub-facet', filterSet);
      var facetChecked = facet.is(':checked');

      if(facetChecked) {
        if (filterSet.hasClass('js-filter-show-only')) {
          return;
        }
        subFacetsInFilterSet.attr('checked', 'checked');
      } else {
        subFacetsInFilterSet.attr('checked', false);
      }
    }

    // Sub-Facet changed
    subFacets.on('change', function(event) {
      var subFacet = $(event.target);
      updateFacet(subFacet);
      if(subFacet.attr('checked')) {
        clearLink.show();
      }
      hideClearLink();
    });

    function updateFacet(subFacet) {
      var filterSet = subFacet.closest('.js-filter');
      var facet = $('.js-filter__facet', filterSet);

      var subFacetsChecked = $('.js-filter__sub-facet:checked', filterSet);
      if(subFacetsChecked.length === 0) {
        if (filterSet.hasClass('js-filter-show-only')) {
          return;
        }
        facet.attr('checked', false);
      } else {
        facet.attr('checked', 'checked');
      }
    }

    // Clear link clicked
    clearLink.on('click', function(event){
      event.preventDefault();
      facets.attr('checked', false);
      subFacets.attr('checked', false);
      clearLink.hide();
    });

    // Miscellaneous functions
    function hideClearLink() {
      var facetsChecked = $('.js-filter__facet:checked', section);
      if(facetsChecked.length === 0) {
        clearLink.hide();
      }
    }
  });
})(jQuery);
